@@ -40,6 +40,17 @@ pub use self::line_iterator::LineIterator;
 pub struct Line<N: Num = f32>(pub Point<N>, pub Point<N>);
 
 impl<N: Num> Line<N> {
+    pub fn new_from_points(points: &[N]) -> Self {
+        if points.len() != 4 {
+            panic!(
+                "Given points slice which isn't exactly 4 elements, length given ... {}",
+                points.len()
+            );
+        }
+
+        Line(Point(points[0], points[1]), Point(points[2], points[3]))
+    }
+
     pub fn new_zero_value() -> Self {
         Line(Point::new_zero_value(), Point::new_zero_value())
     }
@@ -135,6 +146,22 @@ impl<N: Num> Line<N> {
         Self(self.start().max(other.start()), self.end().max(other.end()))
     }
 
+    pub fn min_x(self) -> N {
+        self.start().x().min(self.end().x())
+    }
+
+    pub fn min_y(self) -> N {
+        self.start().y().min(self.end().y())
+    }
+
+    pub fn max_x(self) -> N {
+        self.start().x().max(self.end().x())
+    }
+
+    pub fn max_y(self) -> N {
+        self.start().y().max(self.end().y())
+    }
+
     pub fn overlaps_line(self, other: Line<N>) -> bool {
         self.intersect_line(other).is_some()
     }
