@@ -1,3 +1,4 @@
+mod from_rounded;
 mod half;
 mod num;
 mod num_identity;
@@ -7,6 +8,7 @@ mod number_extensions;
 mod to_rounded;
 mod to_signed_clamped;
 
+pub use self::from_rounded::*;
 pub use self::half::*;
 pub use self::num::*;
 pub use self::num_identity::*;
